// Simple Rain Toy
// Composed for the edification of students by Brian Moriarty
// Released under GLPL-3.0

// The following comment line is for JSLint. Don't remove it!

/*global PS */

// Global namespace variable (RAIN)
// All game-specific global constants, variables and functions are defined as named elements of the RAIN table
// This prevents any namespace clashes with Perlenspiel functions

// Notice how constants are all defined in upper-case
// This makes it easier to remember that you're dealing with a constant

var RAIN = {
	GRID_WIDTH: 24, // width of grid
	GRID_HEIGHT: 24, // height of grid
	BOTTOM_ROW: 23, // last row of grid
	FRAME_RATE: 6,	// animation frame rate; 6/60ths = 10 fps
	BG_COLOR: 0x8080FF, // background color
	DROP_COLOR: 0x4040FF, // raindrop color

// These arrays store the x and y positions of the active raindrops (initially empty)
// Notice how variables in the global namespace always begin with an upper-case character
// This convention makes it easier to distinguish them from lower-case local variables

	DropsX: [],
	DropsY: []
};

// Initialize the toy

PS.Init = function ( options )
{
	"use strict"; // include this line at the beginning of ALL functions
	var y; // local variables are all lower-case

	// Set up the grid

	PS.GridSize( RAIN.GRID_WIDTH, RAIN.GRID_HEIGHT );

	// Turn off bead flashing on all EXCEPT the bottom row
	// We'll use flashing there to get a "free" splash effect

	for ( y = 0; y < RAIN.BOTTOM_ROW; y += 1 )
	{
		PS.BeadFlash( PS.ALL, y, false ); // turns off flashing on entire row
	}	

	// Change background color	

	PS.GridBGColor( RAIN.BG_COLOR );

	// Hide all bead borders

	PS.BeadBorderWidth( PS.ALL, PS.ALL, 0 );

	// preload audio files

	PS.AudioLoad( "fx_drip1" );
	PS.AudioLoad( "fx_drip2" );

	// Set color and text of title

	PS.StatusColor( PS.COLOR_WHITE );
	PS.StatusText( "Simple Rain Toy" );

	// Start the animation timer

	PS.Clock( RAIN.FRAME_RATE );
};

// Called when a bead is clicked

PS.Click = function ( x, y, data, options )
{
	"use strict";

	// Add x & y position of raindrop to animation list

	RAIN.DropsX.push( x );
	RAIN.DropsY.push( y );

	// If drop is above bottom row, start a drop	

	if ( y < RAIN.BOTTOM_ROW )
	{				
		PS.BeadColor( x, y, RAIN.DROP_COLOR ); // set the color
		PS.AudioPlay( "fx_drip1" ); // play drip sound
	}

	// Otherwise splash it immediately

	else
	{
		RAIN.Splash( x, y );
	}

	// Our PS.Tick animation code does the rest!
};

// Splash a raindrop at x/y

RAIN.Splash = function ( x, y )
{
	"use strict";

	// Paint using background color
	// Flashing on bottom row gives us a "free" splash-and-disappear effect
		
	PS.BeadColor( x, y, RAIN.BG_COLOR );	
		
	// Play splash sound
		
	PS.AudioPlay( "fx_drip2" );
};

// Called on every clock tick
// Used to animate the raindrops

PS.Tick = function ( options )
{
	"use strict";
	var len, i, x, y;

	// The length of the RAIN.DropsX array is the current number of drops

	len = RAIN.DropsX.length; // number of drops

	// loop through each active raindrop

	i = 0;
	while ( i < len )
	{
		// get current position of raindrop

		x = RAIN.DropsX[i];
		y = RAIN.DropsY[i];

		// If bead is above last row, erase it and redraw one bead lower

		if ( y < RAIN.BOTTOM_ROW )
		{
			// erase the existing drop

			PS.BeadColor( x, y, RAIN.BG_COLOR );

			// add 1 to y position

			y += 1;

			// update its y position in the array

			RAIN.DropsY[i] = y;

			// Has drop reached the bottom row yet?

			if ( y < RAIN.BOTTOM_ROW ) // nope
			{
				// Repaint the drop one bead lower
				
				PS.BeadColor( x, y, RAIN.DROP_COLOR );
			}

			// Drop has reached bottom! Splash it

			else
			{
				RAIN.Splash( x, y );
			}

			// point index to next drop

			i += 1;	
		}

		// Bead is on last row

		else
		{
			// Remove this drop from x/y arrays

			RAIN.DropsX.splice( i, 1 );			
			RAIN.DropsY.splice( i, 1 );

			// Arrays are now one element smaller, so update the array length variable
			// But leave the index variable [i] alone!
			// It's already pointing at the next drop

			len -= 1;	
		}	
	}
};

// These functions aren't used in this toy
// But they're included to avoid startup warnings

PS.Release = function (x, y, data, options)
{
	"use strict";
};

PS.Enter = function (x, y, data, options)
{
	"use strict";
};

PS.Leave = function (x, y, data, options)
{
	"use strict";
};

PS.KeyDown = function (key, shift, ctrl, options)
{
	"use strict";
};

PS.KeyUp = function (key, shift, ctrl, options)
{
	"use strict";
};

PS.Wheel = function (delta, options)
{
	"use strict";
};



